unit Main;

{$MODE Delphi}

interface

uses
  Controls, Forms, Classes,
  IdBaseComponent, IdComponent, IdTCPServer, IdContext,
  SysUtils, LResources;

type
  TformMain = class(TForm)
    IdTCPServer1: TIdTCPServer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure IdTCPServer1Connect(AContext: TIdContext);
    procedure IdTCPServer1Execute(AContext: TIdContext);
  private
  protected
    FZipCodeList: TStrings;
  public
  end;

  TUserData = class
  public
    FRequestCount: Integer;
  end;

var
  formMain: TformMain;

implementation

uses
  IdGlobal;

procedure TformMain.FormCreate(Sender: TObject);
begin
  IdTCPServer1.bindings[0].SetPeer('127.0.0.1',6000,id_IPV4);
  FZipCodeList := TStringList.Create;
  FZipCodeList.LoadFromFile(includetrailingpathdelimiter(ExtractFilePath(Application.EXEName)) +  includetrailingpathdelimiter('..')+'ZipCodes.dat');
end;

procedure TformMain.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FZipCodeList);
end;

procedure TformMain.IdTCPServer1Connect(AContext: TIdContext);
begin
  AContext.Connection.IOHandler.WriteLn('200 Zip Code Server Ready.');
  AContext.Data := TUserData.Create;
end;

procedure TformMain.IdTCPServer1Execute(AContext: TIdContext);
var
  LCmd: string;
  LCity: string;
  LLine: string;
  LZipCode: string;
begin
  with AContext.Connection.IOHandler do try
    LLine := ReadLn;
    LCmd := Fetch(LLine);
    if AnsiSameText(LCmd, 'QUIT') then begin
      WriteLn('202-Paka!');
      WriteLn('202 ' + IntToStr(TUserData(AContext.Data).FRequestCount)
       + ' requests processed.');
      AContext.Connection.Disconnect;
    end else if AnsiSameText(LCmd, 'Lookup') then begin
      WriteLn('201 Data follows');
      LZipCode := Fetch(LLine);
      while LZipCode <> '' do begin
        LCity := FZipCodeList.Values[LZipCode];
        if LCity <> '' then begin
          Inc(TUserData(AContext.Data).FRequestCount);
          WriteLn(LZipCode + ': ' + LCity);
        end;
        LZipCode := Fetch(LLine);
      end;
      WriteLn('.');
    end else if AnsiSameText(LCmd, 'Help') then begin
      WriteBufferOpen; try
        WriteLn('100 Help follows');
        WriteLn('Help');
        WriteLn('  Provides a list of supported commands.');
        WriteLn('Lookup <Post Code 1> <Post Code 2> ...');
        WriteLn('  Looks up and provides associated cities for specified zip codes.');
        WriteLn('Quit');
        WriteLn('  Signals server that a disconnect is requested.');
        WriteLn('.');
      finally WriteBufferClose; end;
    end else begin
      WriteLn('400 Unknown Command');
    end;
  except
    on E: Exception do begin
      WriteLn('500-Unknown Internal Error');
      WriteLn('500 ' + StringReplace(E.Message, EOL, ' ', [rfReplaceAll]));
      raise;
    end;
  end;
end;

initialization
  {$i main.lrs}

end.
